@@ -492,7 +492,7 @@ static inline bool isdigit(int what, unsigned int flgs)
 #define STATS_INC_WORDCHARS
 #endif
 
-vector<CharFlags> splitFlags = {
+vector<CharFlags> splitFlags{
     {TextSplit::TXTS_NOSPANS, "nospans"},
     {TextSplit::TXTS_ONLYSPANS, "onlyspans"},
     {TextSplit::TXTS_KEEPWILD, "keepwild"}
@@ -530,7 +530,7 @@ bool TextSplit::text_to_words(const string &in)
 	nonalnumcnt++;
 
 	if (c == (unsigned int)-1) {
-	    LOGERR("Textsplit: error occured while scanning UTF-8 string\n");
+	    LOGERR("Textsplit: error occurred while scanning UTF-8 string\n");
 	    return false;
 	}
 
