@@ -18,7 +18,6 @@
 #define _beaglequeue_h_included_
 
 #include <list>
-using std::list;
 
 /**
  * Process the Beagle indexing queue. 
@@ -56,11 +55,11 @@ public:
 
     /** Index a list of files. No db cleaning or stemdb updating. 
      *  Used by the real time monitor */
-    bool indexFiles(list<string>& files);
+    bool indexFiles(std::list<std::string>& files);
     /** Purge a list of files. No way to do this currently and dont want
      *  to do anything as this is mostly called by the monitor when *I* delete
      *  files inside the queue dir */
-    bool purgeFiles(list<string>& files) {return true;}
+    bool purgeFiles(std::list<std::string>& files) {return true;}
 
     /** Called when indexing data from the cache, and from internfile for
      * search result preview */
