@@ -58,6 +58,7 @@ using namespace std;
 #endif
 #include "execmd.h"
 #include "checkretryfailed.h"
+#include "idxstatus.h"
 
 // Command line options
 static int     op_flags;
@@ -141,6 +142,7 @@ class MyUpdater : public DbIxStatusUpdater {
 	    m_file.set("dbtotdocs", status.dbtotdocs);
 	    m_file.set("totfiles", status.totfiles);
 	    m_file.set("fn", status.fn);
+            m_file.set("hasmonitor", status.hasmonitor);
             m_file.holdWrites(false);
 	}
 
@@ -201,6 +203,18 @@ void rclIxIonice(const RclConfig *config)
 #endif
 }
 
+static void setMyPriority(const RclConfig *config)
+{
+#ifndef _WIN32
+    if (setpriority(PRIO_PROCESS, 0, 20) != 0) {
+        LOGINFO("recollindex: can't setpriority(), errno " << errno << "\n");
+    }
+    // Try to ionice. This does not work on all platforms
+    rclIxIonice(config);
+#endif
+}
+
+
 class MakeListWalkerCB : public FsTreeWalkerCB {
 public:
     MakeListWalkerCB(list<string>& files, const vector<string>& selpats)
@@ -316,10 +330,33 @@ static bool checktopdirs(RclConfig *config, vector<string>& nonexist)
     vector<string> tdl;
     if (!config->getConfParam("topdirs", &tdl)) {
         cerr << "No 'topdirs' parameter in configuration\n";
-        LOGERR("recollindex:No 'topdirs' parameter in configuration\n");;
+        LOGERR("recollindex:No 'topdirs' parameter in configuration\n");
         return false;
     }
 
+    // If a restricted list for real-time monitoring exists check that
+    // all entries are descendants from a topdir
+    vector<string> mondirs;
+    if (config->getConfParam("monitordirs", &mondirs)) {
+        for (const auto& sub : mondirs) {
+            bool found{false};
+            for (const auto& top : tdl) {
+                if (path_isdesc(top, sub)) {
+                    found = true;
+                    break;
+                }
+            }
+            if (!found) {
+                string s("Real time monitoring directory entry " + sub +
+                         " is not part of the topdirs tree\n");
+                cerr << s;
+                LOGERR(s);
+                return false;
+            }
+        }
+    }
+
+    
     for (vector<string>::iterator it = tdl.begin(); it != tdl.end(); it++) {
 	*it = path_tildexpand(*it);
         if (!it->size() || !path_isabsolute(*it)) {
@@ -448,12 +485,32 @@ Usage(FILE *where = stderr)
 
 static RclConfig *config;
 
-void lockorexit(Pidfile *pidfile)
+static void lockorexit(Pidfile *pidfile, RclConfig *config)
 {
     pid_t pid;
     if ((pid = pidfile->open()) != 0) {
-	cerr << "Can't become exclusive indexer: " << pidfile->getreason() << 
-	    ". Return (other pid?): " << pid << endl;
+        if (pid > 0) {
+            cerr << "Can't become exclusive indexer: " << pidfile->getreason()
+                 << ". Return (other pid?): " << pid << endl;
+#ifndef _WIN32
+            // Have a look at the status file. If the other process is
+            // a monitor we can tell it to start an incremental pass
+            // by touching the configuration file
+            DbIxStatus status;
+            readIdxStatus(config, status);
+            if (status.hasmonitor) {
+                string cmd("touch ");
+                string path = path_cat(config->getConfDir(), "recoll.conf");
+                cmd += path;
+                system(cmd.c_str());
+                cerr << "Monitoring indexer process was notified of "
+                    "indexing request\n";
+            }
+#endif
+        } else {
+            cerr << "Can't become exclusive indexer: " << pidfile->getreason()
+                 << endl;
+        }            
 	exit(1);
     }
     if (pidfile->write_pid() != 0) {
@@ -567,8 +624,10 @@ int main(int argc, char **argv)
     }
 
     string reason;
-    RclInitFlags flags = (op_flags & OPT_m) && !(op_flags&OPT_D) ? 
-	RCLINIT_DAEMON : RCLINIT_IDX;
+    int flags = RCLINIT_IDX;
+    if ((op_flags & OPT_m) && !(op_flags&OPT_D)) {
+	flags |= RCLINIT_DAEMON;
+    }
     config = recollinit(flags, cleanup, sigcleanup, reason, &a_config);
     if (config == 0 || !config->ok()) {
         addIdxReason("init", reason);
@@ -639,14 +698,8 @@ int main(int argc, char **argv)
     // Log something at LOGINFO to reset the trace file. Else at level
     // 3 it's not even truncated if all docs are up to date.
     LOGINFO("recollindex: starting up\n");
-#ifndef _WIN32
-    if (setpriority(PRIO_PROCESS, 0, 20) != 0) {
-        LOGINFO("recollindex: can't setpriority(), errno " << errno << "\n");
-    }
-    // Try to ionice. This does not work on all platforms
-    rclIxIonice(config);
-#endif
-
+    setMyPriority(config);
+    
     if (op_flags & OPT_r) {
 	if (argc != 1) 
 	    Usage();
@@ -659,7 +712,7 @@ int main(int argc, char **argv)
         flushIdxReasons();
         exit(status ? 0 : 1);
     } else if (op_flags & (OPT_i|OPT_e)) {
-	lockorexit(&pidfile);
+	lockorexit(&pidfile, config);
 
 	list<string> filenames;
 
@@ -717,7 +770,10 @@ int main(int argc, char **argv)
     } else if (op_flags & OPT_m) {
 	if (argc != 0) 
 	    Usage();
-	lockorexit(&pidfile);
+	lockorexit(&pidfile, config);
+        if (updater) {
+	    updater->status.hasmonitor = true;
+        }
 	if (!(op_flags&OPT_D)) {
 	    LOGDEB("recollindex: daemonizing\n");
 #ifndef _WIN32
@@ -732,15 +788,9 @@ int main(int argc, char **argv)
 	}
 	// Need to rewrite pid, it changed
 	pidfile.write_pid();
-#ifndef _WIN32
         // Not too sure if I have to redo the nice thing after daemon(),
         // can't hurt anyway (easier than testing on all platforms...)
-        if (setpriority(PRIO_PROCESS, 0, 20) != 0) {
-            LOGINFO("recollindex: can't setpriority(), errno " << errno<< "\n");
-        }
-	// Try to ionice. This does not work on all platforms
-	rclIxIonice(config);
-#endif
+        setMyPriority(config);
 
 	if (sleepsecs > 0) {
 	    LOGDEB("recollindex: sleeping " << sleepsecs << "\n");
@@ -753,6 +803,7 @@ int main(int argc, char **argv)
 	      }
 	    }
 	}
+
 	if (!(op_flags & OPT_n)) {
 	    makeIndexerOrExit(config, inPlaceReset);
 	    LOGDEB("Recollindex: initial indexing pass before monitoring\n");
@@ -776,10 +827,11 @@ int main(int argc, char **argv)
 	    // Note that -n will be inside the reexec when we come
 	    // back, but the monitor will explicitely strip it before
 	    // starting a config change exec to ensure that we do a
-	    // purging pass in this case.
+	    // purging pass in this latter case (full restart).
 	    o_reexec->reexec();
 #endif
 	}
+
         if (updater) {
 	    updater->status.phase = DbIxStatus::DBIXS_MONITOR;
 	    updater->status.fn.clear();
@@ -801,7 +853,7 @@ int main(int argc, char **argv)
         cerr << "Not yet" << endl;
         return 1;
     } else {
-	lockorexit(&pidfile);
+	lockorexit(&pidfile, config);
 	makeIndexerOrExit(config, inPlaceReset);
 	bool status = confindexer->index(rezero, ConfIndexer::IxTAll, 
                                          indexerFlags);
