@@ -407,7 +407,7 @@ p, li { white-space: pre-wrap; }
         <translation type="unfinished"></translation>
     </message>
     <message>
-        <source>&quot;history&quot; file is damaged or un(read)writeable, please check or remove it: </source>
+        <source>&quot;history&quot; file is damaged, please check or remove it: </source>
         <translation type="unfinished"></translation>
     </message>
 </context>
@@ -1113,6 +1113,18 @@ Please check the desktop file</source>
         <source>Types list empty: maybe wait for indexing to progress?</source>
         <translation type="unfinished"></translation>
     </message>
+    <message>
+        <source>Viewer command line for %1 specifies parent file but URL is http[s]: unsupported</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Tools</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Results</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>RclMainBase</name>
@@ -2349,10 +2361,6 @@ This will help searching very big text  files (ie: log files).</source>
 </context>
 <context>
     <name>uiPrefsDialogBase</name>
-    <message>
-        <source>User preferences</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>User interface</source>
         <translation type="unfinished"></translation>
@@ -2658,5 +2666,13 @@ The default value is 2 (percent). </source>
         <source>Highlight CSS style for query terms</source>
         <translation type="unfinished"></translation>
     </message>
+    <message>
+        <source>Recoll - User Preferences</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Set path translations for the selected index or for the main one if no selection exists.</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 </TS>
