@@ -446,7 +446,7 @@ public:
     void paint(QPainter *painter, const QStyleOptionViewItem &option, 
 	       const QModelIndex &index) const
     {
-	QStyleOptionViewItemV4 opt = option;
+	QStyleOptionViewItem opt = option;
 	initStyleOption(&opt, index);
 	QVariant value = index.data(Qt::DisplayRole);
 	if (value.isValid() && !value.isNull()) {
