@@ -138,9 +138,14 @@ void UIPrefsDialog::setFromPrefs()
 
     keepSortCB->setChecked(prefs.keepSort);
     showTrayIconCB->setChecked(prefs.showTrayIcon);
+    if (!prefs.showTrayIcon) {
+        prefs.closeToTray = false;
+    }
+    closeToTrayCB->setEnabled(showTrayIconCB->checkState());
     closeToTrayCB->setChecked(prefs.closeToTray);
     showTempFileWarningCB->setChecked(prefs.showTempFileWarning == -1);
     previewHtmlCB->setChecked(prefs.previewHtml);
+    previewActiveLinksCB->setChecked(prefs.previewActiveLinks);
     switch (prefs.previewPlainPre) {
     case PrefsPack::PP_BR:
 	plainBRRB->setChecked(1);
@@ -326,6 +331,7 @@ void UIPrefsDialog::accept()
     prefs.showTempFileWarning = showTempFileWarningCB->isChecked() ?
         -1 : 1024;
     prefs.previewHtml = previewHtmlCB->isChecked();
+    prefs.previewActiveLinks = previewActiveLinksCB->isChecked();
 
     if (plainBRRB->isChecked()) {
 	prefs.previewPlainPre = PrefsPack::PP_BR;
@@ -574,6 +580,9 @@ static bool samedir(const string& dir1, const string& dir2)
 
 void UIPrefsDialog::on_showTrayIconCB_clicked()
 {
+    if (!showTrayIconCB->checkState()) {
+        closeToTrayCB->setChecked(false);
+    }
     closeToTrayCB->setEnabled(showTrayIconCB->checkState());
 }
 
