@@ -15,7 +15,7 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-#include "debuglog.h"
+#include "log.h"
 #include "filtseq.h"
 #include "rclconfig.h"
 
@@ -23,20 +23,19 @@ using std::string;
 
 static bool filter(const DocSeqFiltSpec& fs, const Rcl::Doc *x)
 {
-    LOGDEB2(("  Filter: ncrits %d\n", fs.crits.size()));
+    LOGDEB2("  Filter: ncrits "  << (fs.crits.size()) << "\n" );
     // Compare using each criterion in term. We're doing an or:
     // 1st ok ends 
     for (unsigned int i = 0; i < fs.crits.size(); i++) {
 	switch (fs.crits[i]) {
 	case DocSeqFiltSpec::DSFS_MIMETYPE:
-	    LOGDEB2((" filter: MIMETYPE: me [%s] doc [%s]\n",
-		    fs.values[i].c_str(), x->mimetype.c_str()));
+	    LOGDEB2(" filter: MIMETYPE: me ["  << (fs.values[i]) << "] doc ["  << (x->mimetype) << "]\n" );
 	    if (x->mimetype == fs.values[i])
 		return true;
 	    break;
 	case DocSeqFiltSpec::DSFS_QLANG:
 	{
-	    LOGDEB((" filter: QLANG [%s]!!\n", fs.values[i].c_str()));
+	    LOGDEB(" filter: QLANG ["  << (fs.values[i]) << "]!!\n" );
 	}
 	break;
 	case DocSeqFiltSpec::DSFS_PASSALL:
@@ -47,7 +46,7 @@ static bool filter(const DocSeqFiltSpec& fs, const Rcl::Doc *x)
     return false;
 } 
 
-DocSeqFiltered::DocSeqFiltered(RclConfig *conf, STD_SHARED_PTR<DocSequence> iseq, 
+DocSeqFiltered::DocSeqFiltered(RclConfig *conf, std::shared_ptr<DocSequence> iseq, 
 			       DocSeqFiltSpec &filtspec)
     :  DocSeqModifier(iseq), m_config(conf)
 {
@@ -56,7 +55,7 @@ DocSeqFiltered::DocSeqFiltered(RclConfig *conf, STD_SHARED_PTR<DocSequence> iseq
 
 bool DocSeqFiltered::setFiltSpec(const DocSeqFiltSpec &filtspec)
 {
-    LOGDEB0(("DocSeqFiltered::setFiltSpec\n"));
+    LOGDEB0("DocSeqFiltered::setFiltSpec\n" );
     for (unsigned int i = 0; i < filtspec.crits.size(); i++) {
 	switch (filtspec.crits[i]) {
 	case DocSeqFiltSpec::DSFS_MIMETYPE:
@@ -74,7 +73,7 @@ bool DocSeqFiltered::setFiltSpec(const DocSeqFiltSpec &filtspec)
 		m_config->getMimeCatTypes(catg, tps);
 		for (vector<string>::const_iterator it = tps.begin();
 		     it != tps.end(); it++) {
-		    LOGDEB2(("Adding mime: [%s]\n", it->c_str()));
+		    LOGDEB2("Adding mime: ["  << (it) << "]\n" );
 		    m_spec.orCrit(DocSeqFiltSpec::DSFS_MIMETYPE, *it);
 		}
 	    }
@@ -94,7 +93,7 @@ bool DocSeqFiltered::setFiltSpec(const DocSeqFiltSpec &filtspec)
 
 bool DocSeqFiltered::getDoc(int idx, Rcl::Doc &doc, string *)
 {
-    LOGDEB2(("DocSeqFiltered::getDoc() fetching %d\n", idx));
+    LOGDEB2("DocSeqFiltered::getDoc() fetching "  << (idx) << "\n" );
 
     if (idx >= (int)m_dbindices.size()) {
 	// Have to fetch docs and filter until we get enough or
@@ -122,3 +121,4 @@ bool DocSeqFiltered::getDoc(int idx, Rcl::Doc &doc, string *)
     }
     return true;
 }
+
