@@ -1,4 +1,4 @@
-/* Copyright (C) 2004 J.F.Dockes
+/* Copyright (C) 2004-2018 J.F.Dockes
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
  *   the Free Software Foundation; either version 2 of the License, or
@@ -50,6 +50,7 @@ using namespace std;
 #include "searchdata.h"
 #include "rclquery.h"
 #include "rclquery_p.h"
+#include "rclvalues.h"
 #include "md5ut.h"
 #include "rclversion.h"
 #include "cancelcheck.h"
@@ -58,14 +59,27 @@ using namespace std;
 #include "rclinit.h"
 #include "internfile.h"
 #include "utf8fn.h"
+#include "wipedir.h"
 #ifdef RCL_USE_ASPELL
 #include "rclaspell.h"
 #endif
+#include "zlibut.h"
+
+#ifndef XAPIAN_AT_LEAST
+// Added in Xapian 1.4.2. Define it here for older versions
+#define XAPIAN_AT_LEAST(A,B,C) \
+ (XAPIAN_MAJOR_VERSION > (A) || \
+ (XAPIAN_MAJOR_VERSION == (A) && \
+ (XAPIAN_MINOR_VERSION > (B) || \
+ (XAPIAN_MINOR_VERSION == (B) && XAPIAN_REVISION >= (C)))))
+#endif
+
 
 // Recoll index format version is stored in user metadata. When this change,
 // we can't open the db and will have to reindex.
 static const string cstr_RCL_IDX_VERSION_KEY("RCL_IDX_VERSION_KEY");
 static const string cstr_RCL_IDX_VERSION("1");
+static const string cstr_RCL_IDX_DESCRIPTOR_KEY("RCL_IDX_DESCRIPTOR_KEY");
 
 static const string cstr_mbreaks("rclmbreaks");
 
@@ -189,8 +203,8 @@ void *DbUpdWorker(void* vdbp)
 	switch (tsk->op) {
 	case DbUpdTask::AddOrUpdate:
 	    LOGDEB("DbUpdWorker: got add/update task, ql " << qsz << "\n");
-	    status = ndbp->addOrUpdateWrite(tsk->udi, tsk->uniterm, 
-					    tsk->doc, tsk->txtlen);
+	    status = ndbp->addOrUpdateWrite(
+                tsk->udi, tsk->uniterm, tsk->doc, tsk->txtlen, tsk->rawztext);
 	    break;
 	case DbUpdTask::Delete:
 	    LOGDEB("DbUpdWorker: got delete task, ql " << qsz << "\n");
@@ -237,9 +251,100 @@ void Db::Native::maybeStartThreads()
 
 #endif // IDX_THREADS
 
+void Db::Native::openWrite(const string& dir, Db::OpenMode mode)
+{
+    int action = (mode == Db::DbUpd) ? Xapian::DB_CREATE_OR_OPEN :
+        Xapian::DB_CREATE_OR_OVERWRITE;
+
+#ifdef _WIN32
+    // Xapian is quite bad at erasing partial db which can
+    // occur because of open file deletion errors on
+    // Windows. 
+    if (mode == DbTrunc) {
+        if (path_exists(path_cat(dir, "iamchert"))) {
+            wipedir(dir);
+            unlink(dir.c_str());
+        }
+    }
+#endif
+    
+    if (::access(dir.c_str(), 0) == 0) {
+        // Existing index
+        xwdb = Xapian::WritableDatabase(dir, action);
+    } else {
+        // New index. If possible, and depending on config, use a stub
+        // to force using Chert. No sense in doing this if we are
+        // storing the text anyway.
+#if XAPIAN_AT_LEAST(1,3,0) && XAPIAN_HAS_CHERT_BACKEND
+        // Xapian with Glass and Chert support. If storedoctext is
+        // specified in the configuration, use the default backend
+        // (Glass), else force Chert. There might be reasons why
+        // someone would want to use Chert and store text anyway, but
+        // it's an exotic case, and things are complicated enough
+        // already.
+        if (o_index_storedoctext) {
+            xwdb = Xapian::WritableDatabase(dir, action);
+            m_storetext = true;
+        } else {
+            // Force Chert format, don't store the text.
+            string stub = path_cat(m_rcldb->m_config->getConfDir(),
+                                   "xapian.stub");
+            FILE *fp = fopen(stub.c_str(), "w");
+            if (nullptr == fp) {
+                throw(string("Can't create ") + stub);
+            }
+            fprintf(fp, "chert %s\n", dir.c_str());
+            fclose(fp);
+            xwdb = Xapian::WritableDatabase(stub, action);
+            m_storetext = false;
+        }
+        LOGINF("Rcl::Db::openWrite: new index will " << (m_storetext?"":"not ")
+               << "store document text\n");
+#else
+        // Old Xapian (chert only) or newer (no chert). Use the
+        // default index backend and let the user decide of the
+        // abstract generation method. The configured default is to
+        // store the text.
+        xwdb = Xapian::WritableDatabase(dir, action);
+        m_storetext = o_index_storedoctext;
+#endif
+        // Set the storetext value inside the index descriptor (new
+        // with recoll 1.24, maybe we'll have other stuff to store in
+        // there in the future).
+        string desc = string("storetext=") + (m_storetext ? "1" : "0") + "\n";
+        xwdb.set_metadata(cstr_RCL_IDX_DESCRIPTOR_KEY, desc);
+    }
+    
+    // If the index is empty, write the data format version at once
+    // to avoid stupid error messages:
+    if (xwdb.get_doccount() == 0) {
+        xwdb.set_metadata(cstr_RCL_IDX_VERSION_KEY, cstr_RCL_IDX_VERSION);
+    }
+
+    m_iswritable = true;
+
+#ifdef IDX_THREADS
+    maybeStartThreads();
+#endif
+}
+
+void Db::Native::openRead(const string& dir)
+{
+    m_iswritable = false;
+    xrdb = Xapian::Database(dir);
+    string desc = xrdb.get_metadata(cstr_RCL_IDX_DESCRIPTOR_KEY);
+    ConfSimple cf(desc, 1);
+    string val;
+    m_storetext = false;
+    if (cf.get("storetext", val) && stringToBool(val)) {
+        m_storetext = true;
+    }
+    LOGDEB("Db::openRead: index " << (m_storetext?"stores":"does not store") <<
+           " document text\n");
+}
+
 /* See comment in class declaration: return all subdocuments of a
- * document given by its unique id. 
-*/
+ * document given by its unique id. */
 bool Db::Native::subDocs(const string &udi, int idxi, 
 			 vector<Xapian::docid>& docids) 
 {
@@ -440,7 +545,7 @@ Xapian::docid Db::Native::getDoc(const string& udi, int idxi,
 
 // Turn data record from db into document fields
 bool Db::Native::dbDataToRclDoc(Xapian::docid docid, std::string &data, 
-				Doc &doc)
+				Doc &doc, bool fetchtext)
 {
     LOGDEB2("Db::dbDataToRclDoc: data:\n" << data << "\n");
     ConfSimple parms(data);
@@ -500,6 +605,9 @@ bool Db::Native::dbDataToRclDoc(Xapian::docid docid, std::string &data,
     }
     doc.meta[Doc::keyurl] = doc.url;
     doc.meta[Doc::keymt] = doc.dmtime.empty() ? doc.fmtime : doc.dmtime;
+    if (fetchtext) {
+        getRawText(docid, doc.text);
+    }
     return true;
 }
 
@@ -579,18 +687,39 @@ int Db::Native::getPageNumberForPosition(const vector<int>& pbreaks, int pos)
     return int(it - pbreaks.begin() + 1);
 }
 
+bool Db::Native::getRawText(Xapian::docid docid, string& rawtext)
+{
+    if (!m_storetext) {
+        LOGDEB("Db::Native::getRawText: document text not stored in index\n");
+        return false;
+    }
+    string reason;
+    XAPTRY(rawtext = xrdb.get_metadata(rawtextMetaKey(docid)), xrdb, reason);
+    if (!reason.empty()) {
+        LOGERR("Rcl::Db::getRawText: could not get value: " << reason << endl);
+        return false;
+    }
+    if (rawtext.empty()) {
+        return true;
+    }
+    ZLibUtBuf cbuf;
+    inflateToBuf(rawtext.c_str(), rawtext.size(), cbuf);
+    rawtext.assign(cbuf.getBuf(), cbuf.getCnt());
+    return true;
+}
+
 // Note: we're passed a Xapian::Document* because Xapian
 // reference-counting is not mt-safe. We take ownership and need
 // to delete it before returning.
-bool Db::Native::addOrUpdateWrite(const string& udi, const string& uniterm, 
-				  Xapian::Document *newdocument_ptr, 
-                                  size_t textlen)
+bool Db::Native::addOrUpdateWrite(
+    const string& udi, const string& uniterm, Xapian::Document *newdocument_ptr, 
+    size_t textlen, const string& rawztext)
 {
 #ifdef IDX_THREADS
     Chrono chron;
     std::unique_lock<std::mutex> lock(m_mutex);
 #endif
-    std::shared_ptr<Xapian::Document> doc_cleaner(newdocument_ptr);
+    std::unique_ptr<Xapian::Document> doc_cleaner(newdocument_ptr);
 
     // Check file system full every mbyte of indexed text. It's a bit wasteful
     // to do this after having prepared the document, but it needs to be in
@@ -613,9 +742,9 @@ bool Db::Native::addOrUpdateWrite(const string& udi, const string& uniterm,
     string ermsg;
 
     // Add db entry or update existing entry:
+    Xapian::docid did = 0;
     try {
-	Xapian::docid did = 
-	    xwdb.replace_document(uniterm, *newdocument_ptr);
+	did = xwdb.replace_document(uniterm, *newdocument_ptr);
 	if (did < m_rcldb->updated.size()) {
             // This is necessary because only the file-level docs are tested
             // by needUpdate(), so the subdocs existence flags are only set
@@ -626,7 +755,6 @@ bool Db::Native::addOrUpdateWrite(const string& udi, const string& uniterm,
 	    LOGINFO("Db::add: docid " << did << " added [" << fnc << "]\n");
 	}
     } XCATCHERROR(ermsg);
-
     if (!ermsg.empty()) {
 	LOGERR("Db::add: replace_document failed: " << ermsg << "\n");
 	ermsg.erase();
@@ -642,6 +770,14 @@ bool Db::Native::addOrUpdateWrite(const string& udi, const string& uniterm,
 	}
     }
 
+    XAPTRY(xwdb.set_metadata(rawtextMetaKey(did), rawztext),
+           xwdb, m_rcldb->m_reason);
+    if (!m_rcldb->m_reason.empty()) {
+        LOGERR("Db::addOrUpdate: set_metadata error: " <<
+               m_rcldb->m_reason << "\n");
+        // This only affects snippets, so let's say not fatal
+    }
+    
     // Test if we're over the flush threshold (limit memory usage):
     bool ret = m_rcldb->maybeflush(textlen);
 #ifdef IDX_THREADS
@@ -681,7 +817,7 @@ bool Db::Native::purgeFileWrite(bool orphansOnly, const string& udi,
 	    }
 	} else {
 	    LOGDEB("purgeFile: delete docid " << *docid << "\n");
-	    xwdb.delete_document(*docid);
+            deleteDocument(*docid);
 	}
 	vector<Xapian::docid> docids;
 	subDocs(udi, 0, docids);
@@ -704,7 +840,7 @@ bool Db::Native::purgeFileWrite(bool orphansOnly, const string& udi,
 		
 	    if (!orphansOnly || sig != subdocsig) {
 		LOGDEB("Db::purgeFile: delete subdoc " << *it << "\n");
-		xwdb.delete_document(*it);
+		deleteDocument(*it);
 	    }
 	}
 	return true;
@@ -742,6 +878,7 @@ Db::Db(const RclConfig *cfp)
 	m_config->getConfParam("maxfsoccuppc", &m_maxFsOccupPc);
 	m_config->getConfParam("idxflushmb", &m_flushMb);
 	m_config->getConfParam("idxmetastoredlen", &m_idxMetaStoredLen);
+	m_config->getConfParam("idxtexttruncatelen", &m_idxTextTruncateLen);
     }
 }
 
@@ -753,7 +890,9 @@ Db::~Db()
     LOGDEB("Db::~Db: isopen " << m_ndb->m_isopen << " m_iswritable " <<
            m_ndb->m_iswritable << "\n");
     i_close(true);
+#ifdef RCL_USE_ASPELL
     delete m_aspell;
+#endif
     delete m_config;
 }
 
@@ -764,6 +903,7 @@ vector<string> Db::getStemmerNames()
     return res;
 }
 
+
 bool Db::open(OpenMode mode, OpenError *error)
 {
     if (error)
@@ -790,61 +930,28 @@ bool Db::open(OpenMode mode, OpenError *error)
 	switch (mode) {
 	case DbUpd:
 	case DbTrunc: 
-	    {
-		int action = (mode == DbUpd) ? Xapian::DB_CREATE_OR_OPEN :
-		    Xapian::DB_CREATE_OR_OVERWRITE;
-                if (::access(dir.c_str(), 0) != 0) {
-                    // New index. use a stub to force using Chert
-                    string stub = path_cat(m_config->getConfDir(),
-                                           "xapian.stub");
-                    FILE *fp = fopen(stub.c_str(), "w");
-                    if (nullptr == fp) {
-                        throw(string("Can't create ") + stub);
-                    }
-                    fprintf(fp, "chert %s\n", dir.c_str());
-                    fclose(fp);
-                    m_ndb->xwdb = Xapian::WritableDatabase(stub, action);
-                } else {
-                    m_ndb->xwdb = Xapian::WritableDatabase(dir, action);
-                }
-                // If db is empty, write the data format version at once
-                // to avoid stupid error messages:
-                if (m_ndb->xwdb.get_doccount() == 0)
-                    m_ndb->xwdb.set_metadata(cstr_RCL_IDX_VERSION_KEY, 
-                                             cstr_RCL_IDX_VERSION);
-		m_ndb->m_iswritable = true;
-#ifdef IDX_THREADS
-		m_ndb->maybeStartThreads();
-#endif
-		// We used to open a readonly object in addition to
-		// the r/w one because some operations were faster
-		// when performed through a Database: no forced
-		// flushes on allterms_begin(), used in
-		// subDocs(). This issue has been gone for a long time
-                // (now: Xapian 1.2) and the separate objects seem to
-                // trigger other Xapian issues, so the query db is now
-                // a clone of the update one.
-		m_ndb->xrdb = m_ndb->xwdb;
-		LOGDEB("Db::open: lastdocid: " << m_ndb->xwdb.get_lastdocid() <<
-                       "\n");
-                LOGDEB2("Db::open: resetting updated\n");
-                updated.resize(m_ndb->xwdb.get_lastdocid() + 1);
-                for (unsigned int i = 0; i < updated.size(); i++)
-                    updated[i] = false;
-	    }
+            m_ndb->openWrite(dir, mode);
+            updated = vector<bool>(m_ndb->xwdb.get_lastdocid() + 1, false);
+            // We used to open a readonly object in addition to the
+            // r/w one because some operations were faster when
+            // performed through a Database: no forced flushes on
+            // allterms_begin(), used in subDocs(). This issue has
+            // been gone for a long time (now: Xapian 1.2) and the
+            // separate objects seem to trigger other Xapian issues,
+            // so the query db is now a clone of the update one.
+            m_ndb->xrdb = m_ndb->xwdb;
+            LOGDEB("Db::open: lastdocid: " <<m_ndb->xwdb.get_lastdocid()<<"\n");
 	    break;
 	case DbRO:
 	default:
-	    m_ndb->m_iswritable = false;
-	    m_ndb->xrdb = Xapian::Database(dir);
-	    for (vector<string>::iterator it = m_extraDbs.begin();
-		 it != m_extraDbs.end(); it++) {
+            m_ndb->openRead(dir);
+            for (auto& db : m_extraDbs) {
 		if (error)
 		    *error = DbOpenExtraDb;
-		LOGDEB("Db::Open: adding query db [" << &(*it) << "]\n");
+		LOGDEB("Db::Open: adding query db [" << &db << "]\n");
                 // An error here used to be non-fatal (1.13 and older)
                 // but I can't see why
-                m_ndb->xrdb.add_database(Xapian::Database(*it));
+                m_ndb->xrdb.add_database(Xapian::Database(db));
 	    }
 	    break;
 	}
@@ -1039,13 +1146,17 @@ bool Db::testDbDir(const string &dir, bool *stripped_p)
     LOGDEB("Db::testDbDir: [" << dir << "]\n");
     try {
 	Xapian::Database db(dir);
-	// If we have terms with a leading ':' it's an
-	// unstripped index
-	Xapian::TermIterator term = db.allterms_begin(":");
-	if (term == db.allterms_end())
+	// If the prefix for mimetype is wrapped, it's an unstripped
+	// index. T has been in use in recoll since the beginning and
+	// all documents have a T field (possibly empty).
+	Xapian::TermIterator term = db.allterms_begin(":T:");
+	if (term == db.allterms_end()) {
 	    mstripped = true;
-	else
+        } else {
 	    mstripped = false;
+        }
+        LOGDEB("testDbDir: " << dir << " is a " <<
+               (mstripped ? "stripped" : "raw") << " index\n");
     } XCATCHERROR(aerr);
     if (!aerr.empty()) {
 	LOGERR("Db::Open: error while trying to open database from [" <<
@@ -1358,6 +1469,7 @@ bool Db::addOrUpdate(const string &udi, const string &parent_udi, Doc &doc)
     // Udi unique term: this is used for file existence/uptodate
     // checks, and unique id for the replace_document() call.
     string uniterm = make_uniterm(udi);
+    string rawztext; // Doc compressed text
 
     if (doc.onlyxattr) {
 	// Only updating an existing doc with new extended attributes
@@ -1371,6 +1483,10 @@ bool Db::addOrUpdate(const string &udi, const string &parent_udi, Doc &doc)
         }
     } else {
 
+        if (m_idxTextTruncateLen > 0) {
+            doc.text = truncate_to_word(doc.text, m_idxTextTruncateLen);
+        }
+        
 	// If the ipath is like a path, index the last element. This is
 	// for compound documents like zip and chm for which the filter
 	// uses the file path as ipath. 
@@ -1409,7 +1525,8 @@ bool Db::addOrUpdate(const string &udi, const string &parent_udi, Doc &doc)
 	    for (vector<string>::iterator it = vpath.begin(); 
 		 it != vpath.end(); it++){
 		if (it->length() > 230) {
-		    // Just truncate it. May still be useful because of wildcards
+		    // Just truncate it. May still be useful because
+		    // of wildcards
 		    *it = it->substr(0, 230);
 		}
 		newdocument.add_posting(wrap_prefix(pathelt_prefix) + *it, 
@@ -1424,26 +1541,36 @@ bool Db::addOrUpdate(const string &udi, const string &parent_udi, Doc &doc)
 	//
 	// The order has no importance, and we set a position gap of 100
 	// between fields to avoid false proximity matches.
-	map<string, string>::iterator meta_it;
-	for (meta_it = doc.meta.begin(); meta_it != doc.meta.end(); meta_it++) {
-	    if (!meta_it->second.empty()) {
-		const FieldTraits *ftp;
-		// We don't test for an empty prefix here. Some fields are part
-		// of the internal conf with an empty prefix (ie: abstract).
-		if (!fieldToTraits(meta_it->first, &ftp)) {
-		    LOGDEB0("Db::add: no prefix for field [" <<
-                            meta_it->first << "], no indexing\n");
-		    continue;
-		}
-		LOGDEB0("Db::add: field [" << meta_it->first << "] pfx [" <<
+	for (const auto& entry: doc.meta) {
+	    if (entry.second.empty()) {
+                continue;
+            }
+            const FieldTraits *ftp{nullptr};
+            fieldToTraits(entry.first, &ftp);
+            if (ftp && ftp->valueslot) {
+                LOGDEB("Adding value: for field " << entry.first << " slot "
+                       << ftp->valueslot << endl);
+                add_field_value(newdocument, *ftp, entry.second);
+            }
+
+            // There was an old comment here about not testing for
+            // empty prefix, and we indeed did not test. I don't think
+            // that it makes sense any more (and was in disagreement
+            // with the LOG message. Really now: no prefix: no
+            // indexing.
+            if (ftp && !ftp->pfx.empty()) {
+                LOGDEB0("Db::add: field [" << entry.first << "] pfx [" <<
                         ftp->pfx << "] inc " << ftp->wdfinc << ": [" <<
-                        meta_it->second << "]\n");
+                        entry.second << "]\n");
                 splitter.setTraits(*ftp);
-		if (!splitter.text_to_words(meta_it->second)) {
-		    LOGDEB("Db::addOrUpdate: split failed for " <<
-                           meta_it->first << "\n");
+                if (!splitter.text_to_words(entry.second)) {
+                    LOGDEB("Db::addOrUpdate: split failed for " <<
+                           entry.first << "\n");
                 }
-	    }
+            } else {
+                LOGDEB0("Db::add: no prefix for field [" <<
+                        entry.first << "], no indexing\n");
+            }
 	}
 
         // Reset to no prefix and default params
@@ -1458,8 +1585,15 @@ bool Db::addOrUpdate(const string &udi, const string &parent_udi, Doc &doc)
 #ifdef TEXTSPLIT_STATS
 	splitter.resetStats();
 #endif
-	if (!splitter.text_to_words(doc.text))
+	if (!splitter.text_to_words(doc.text)) {
 	    LOGDEB("Db::addOrUpdate: split failed for main text\n");
+        } else {
+            if (m_ndb->m_storetext) {
+                ZLibUtBuf buf;
+                deflateToBuf(doc.text.c_str(), doc.text.size(), buf);
+                rawztext.assign(buf.getBuf(), buf.getCnt());
+            }
+        }
 
 #ifdef TEXTSPLIT_STATS
 	// Reject bad data. unrecognized base64 text is characterized by
@@ -1489,8 +1623,8 @@ bool Db::addOrUpdate(const string &udi, const string &parent_udi, Doc &doc)
 	if (doc.getmeta(Doc::keyfn, &utf8fn) && !utf8fn.empty()) {
 	    string fn;
 	    if (unacmaybefold(utf8fn, fn, "UTF-8", UNACOP_UNACFOLD)) {
-		// We should truncate after extracting the extension, but this is
-		// a pathological case anyway
+		// We should truncate after extracting the extension,
+		// but this is a pathological case anyway
 		if (fn.size() > 230)
 		    utf8truncate(fn, 230);
 		string::size_type pos = fn.rfind('.');
@@ -1498,7 +1632,7 @@ bool Db::addOrUpdate(const string &udi, const string &parent_udi, Doc &doc)
 		    newdocument.add_boolean_term(wrap_prefix(fileext_prefix) + 
 						 fn.substr(pos + 1));
 		}
-		newdocument.add_term(wrap_prefix(unsplitfilename_prefix) + fn, 0);
+		newdocument.add_term(wrap_prefix(unsplitfilename_prefix) + fn,0);
 	    }
 	}
 
@@ -1675,8 +1809,9 @@ bool Db::addOrUpdate(const string &udi, const string &parent_udi, Doc &doc)
     }
 #ifdef IDX_THREADS
     if (m_ndb->m_havewriteq) {
-	DbUpdTask *tp = new DbUpdTask(DbUpdTask::AddOrUpdate, udi, uniterm, 
-				      newdocument_ptr, doc.text.length());
+	DbUpdTask *tp = new DbUpdTask(
+            DbUpdTask::AddOrUpdate, udi, uniterm, newdocument_ptr,
+            doc.text.length(), rawztext);
 	if (!m_ndb->m_wqueue.put(tp)) {
 	    LOGERR("Db::addOrUpdate:Cant queue task\n");
             delete newdocument_ptr;
@@ -1688,7 +1823,7 @@ bool Db::addOrUpdate(const string &udi, const string &parent_udi, Doc &doc)
 #endif
 
     return m_ndb->addOrUpdateWrite(udi, uniterm, newdocument_ptr,
-				   doc.text.length());
+				   doc.text.length(), rawztext);
 }
 
 bool Db::Native::docToXdocXattrOnly(TextSplitDb *splitter, const string &udi, 
@@ -2052,7 +2187,7 @@ bool Db::purge()
 		    Xapian::termcount trms = m_ndb->xwdb.get_doclength(docid);
 		    maybeflush(trms * 5);
 		}
-		m_ndb->xwdb.delete_document(docid);
+		m_ndb->deleteDocument(docid);
 		LOGDEB("Db::purge: deleted document #" << docid << "\n");
 	    } catch (const Xapian::DocNotFoundError &) {
 		LOGDEB0("Db::purge: document #" << docid << " not found\n");
@@ -2113,8 +2248,9 @@ bool Db::purgeFile(const string &udi, bool *existed)
 
 #ifdef IDX_THREADS
     if (m_ndb->m_havewriteq) {
+        string rztxt;
 	DbUpdTask *tp = new DbUpdTask(DbUpdTask::Delete, udi, uniterm, 
-				      0, (size_t)-1);
+				      0, (size_t)-1, rztxt);
 	if (!m_ndb->m_wqueue.put(tp)) {
 	    LOGERR("Db::purgeFile:Cant queue task\n");
 	    return false;
@@ -2140,8 +2276,9 @@ bool Db::purgeOrphans(const string &udi)
 
 #ifdef IDX_THREADS
     if (m_ndb->m_havewriteq) {
+        string rztxt;
 	DbUpdTask *tp = new DbUpdTask(DbUpdTask::PurgeOrphans, udi, uniterm, 
-				      0, (size_t)-1);
+				      0, (size_t)-1, rztxt);
 	if (!m_ndb->m_wqueue.put(tp)) {
 	    LOGERR("Db::purgeFile:Cant queue task\n");
 	    return false;
